/*
 *	Copyright (C) 2009 by Chebotarev Roman <roma@ultranet.ru>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DHCDROP_H
#define DHCDROP_H


#define RELEASE_DATE        "17.05.2009"
#define PROG_NAME			"DHCP-dropper"
#define BASE_VERSION        PROG_NAME" v0.4"
#define VERSION             BASE_VERSION

#define COPYRIGHT           VERSION" was written by Chebotarev Roman at "RELEASE_DATE"\n"
#define	HOMEPAGE			"Home page: http://www.netpatch.ru/dhcdrop.html\n"

#define DEF_TTL             64
#define IP_HDR_LEN          5       /* IP header len in 32-bit words */
#define B_IP_HDR_LEN        5 * 4   /* IP header len in bytes */
#define DEF_SERVER_PORT     67
#define DEF_CLIENT_PORT     68
#define MAX_PORT_VALUE      65535
#define DEF_TIMEOUT			3
#define MAX_CHILDREN		32
#define	CAP_TIMEOUT			100		/* Waiting 100 microseconds for avoid CPU overload (microsecs) */
#define FAIL_TIMEOUT		2		/* ms */
#define IPTOSBUFFERS		12

#define DHCP_UDP_OVERHEAD   (20 +   /* IP header */           \
                 8)                 /* UDP header */
#define DHCP_SNAME_LEN      64
#define DHCP_FILE_LEN       128
#define DHCP_FIXED_NON_UDP  236
#define DHCP_FIXED_LEN      (DHCP_FIXED_NON_UDP + DHCP_UDP_OVERHEAD)
                        /* Everything but options. */
#define DHCP_MTU_MAX        1500
#define DHCP_OPTION_LEN     (DHCP_MTU_MAX - DHCP_FIXED_LEN)

#define BOOTP_MIN_LEN       300
#define DHCP_MIN_LEN        548

#define MAX_ADDR_COUNT      255
#define MAX_ATTEMPTS        5
#define DEF_HOSTNAME        "DHCP-dropper"
#define DEF_DHCPCLIENT      DEF_HOSTNAME
#define STR_DHCP_FILTER     "(ether dst host %s or ether dst host FF:FF:FF:FF:FF:FF) and udp and src port %d and dst port %d"

/*Defined errors codes*/
#define ERR_EUID                10
#define ERR_SIGNAL              20
#define ERR_CONFIG              30
#define ERR_MEMORY              40
#define ERR_OPENDEV             50
#define	ERR_LISTING_DEV			51
#define ERR_FILTER_OWERFLOW     60
#define ERR_PCAP_COMPILE        70
#define ERR_PCAP_SETFILTER      80
#define ERR_SENDPACKET          90
#define ERR_GETPACKET           100
#define ERR_SETNOBLOCK          110
#define ERR_INVALID_DEV			120
#define ILLEGAL_SERV_FOUND      200

#pragma pack(1)

#ifdef  __FreeBSD__
#define PCAP_OPEN_LIVE_TIMEOUT	1
#elif	__linux__
#define	PCAP_OPEN_LIVE_TIMEOUT	-1
#elif	__WIN32
#define	PCAP_OPEN_LIVE_TIMEOUT	1
#define MAX_THREADS_COUNT	MAXIMUM_WAIT_OBJECTS /* MAXIMUM_WAIT_OBJECTS == 64 (winbase.h) */
#else
#error "Unsupported system! Supported is Linux & FreeBSD & Windows (MinGW)."
#endif

/* UDP header as specified by RFC 768, August 1980. */

struct udphdr
{
	uint16_t source;
	uint16_t dest;
	uint16_t len;
	uint16_t check;
};

/*
 * Structure of an internet header, naked of options.
 */
struct iphdr
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#else
# error "Please fix <bits/endian.h>"
#endif
    uint8_t tos;
    uint16_t tot_len;
    uint16_t id;
    uint16_t frag_off;
    uint8_t ttl;
    uint8_t protocol;
    uint16_t check;
    uint32_t saddr;
    uint32_t daddr;
    /*The options start here. */
};

const uint8_t magic_cookie[]={99, 130, 83, 99};

struct ignored_mac_node
{
    char mac_addr[19];
    struct ignored_mac_node * next;
};

struct config_params
{
    char    test_mode;
    char    yes;
    char    rand_mac_always;
    int     max_attempts;
    char    *if_name;
    int     max_address_count;
    char    client_hostname[64];
    char    dhcp_client[16];
    char    str_source_mac[19];
    char    flood;
    int     children_count;
    char    broadcast;
    int     client_port;
    int     server_port;
    char    always_wait_from_67;
    char    always_wait_to_68;
	int		timeout;
	int		discover;
};


struct full_packet_header
{
    struct ether_header eth_head;
    struct iphdr ip_header;
    struct udphdr udp_header;
};

typedef struct flood_params
{
	struct config_params * params;
	struct dhcp_packet * dhcp_data;
	struct full_packet_header * header;
#ifdef _WIN32
	HANDLE * flood_threads;
	DWORD  * threads_ids;
#endif
} *flood_params_ptr;

#define CLEANUP							0xFF

#ifdef	_WIN32

#define bzero(x, y)						ZeroMemory(x, y)
#define	sleep(x)						Sleep(x * 1000)
#define random							rand
#define srandom							srand
#define	pcap_inject						pcap_sendpacket
#define set_timer_handler(on_timer)
#define	timer_start(ms, on_timer)		timeSetEvent(ms * 1000,\
								10, (LPTIMECALLBACK) on_timer,\
								0, TIME_ONESHOT )
#define timer_stop(t)					timeKillEvent(t)
#define	set_console_handler(handler)	SetConsoleCtrlHandler(handler, 1);
#define INTERRUPT_SIGNAL				CTRL_C_EVENT
#define	CHILDREN_TYPE					"threads"

#else

#define set_console_handler(handler) \
	if(signal(SIGINT, (void*)handler) == SIG_ERR) \
	{ \
		perror("signal"); \
		exit(ERR_SIGNAL); \
	}

#define set_timer_handler(on_timer) \
	if(signal(SIGALRM, (void*)on_timer) == SIG_ERR) \
	{ \
		perror("signal"); \
		exit(ERR_SIGNAL); \
	}
#define	timer_start(x, on_timer)		alarm(x)
#define timer_stop(t)					alarm(0)
#define INTERRUPT_SIGNAL				SIGINT
#define	CHILDREN_TYPE					"children"

#pragma pack(1)

struct pseudo_header
{
    unsigned int src_addr;
    unsigned int dst_addr;
    unsigned char zero ;
    unsigned char proto;
    unsigned short length;
};

#endif

uint32_t to_cidr(const uint32_t mask);
char *iptos(const uint32_t in);
void ifprint(const pcap_if_t *dev);
#ifdef  _WIN32
BOOL WINAPI int_sign(DWORD signal);
#else
int int_sign(int32_t signal);
#endif
unsigned short rs_crc(const unsigned short *buffer, int length);
uint16_t make_dhcp_req
    (
        struct  dhcp_packet * dhcp_data,
        const uint8_t * ether_src_addr,
        const struct in_addr * server_address,
        const struct in_addr * req_addr,
        const int xid,
        const struct config_params * config
    );
int assemble_header
    (
        struct full_packet_header * header,
        const struct in_addr src_addr,
        const struct in_addr dst_addr,
        const unsigned char *data,
        const int data_len,
        const struct config_params * params
    );
uint16_t set_dhcp_type(const struct dhcp_packet *request, const uint16_t new_type);
int get_dhcp_option(const struct dhcp_packet *request, const uint16_t packet_len,
                        const int req_option, void * option_value);
int decode_dhcp(const struct dhcp_packet * packet, const uint16_t packet_len);
#ifdef  _WIN32
void CALLBACK timeout(UINT uTimerID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2);
#else
void timeout(int signal);
#endif
int add_str_mac_to_ignor(struct ignored_mac_node ** start_list, struct ignored_mac_node ** end_list, char * ether_addr);
int add_mac_to_ignor(struct ignored_mac_node ** start_list, struct ignored_mac_node ** end_list, struct ether_header *eptr);
void usage(const char help);
inline void rand_ether_addr(char * str_mac_addr);
char * interfaces_discover(const int num);
void configure(const int argc, char **argv, struct config_params * params);
int send_packet(pcap_t* descr, struct full_packet_header * header,
                const uint8_t *snd_data, const int data_len, const struct config_params * params);
void packet_handler(u_char *out_packet, const struct pcap_pkthdr *h,
                                   const u_char *packet);
#ifdef _WIN32
unsigned __stdcall start_flood( void* arg );
#else
int start_flood( void* arg );
#endif
void new_child(const struct flood_params * p, const int child_num);
void wait_children(const struct flood_params * p);
int send_flood(struct config_params * params,
                struct full_packet_header * header, struct dhcp_packet * dhcp_data);
int dhcp_transaction(pcap_t * pcap_socket,
                    struct full_packet_header * header,
                    struct dhcp_packet *dhcp_data,
                    int * snd_data_len,
                    struct config_params *params,
                    char *str_dropped_mac,
                    char * filter,
                    int filter_len,
                    struct dhcp_packet *dhcp_resp,
                    struct bpf_program *fp
);


#endif	/* ifdef DHCDROP_H*/
